@@ -127,9 +127,10 @@ impl GenerateCI {
         let project_name = pyproject
             .and_then(|project| project.project_name())
             .unwrap_or(&project_layout.extension_name);
+        let sdist = pyproject_toml.is_some();
 
         match self.ci {
-            Provider::GitHub => self.generate_github(project_name, &bridge),
+            Provider::GitHub => self.generate_github(project_name, &bridge, sdist),
         }
     }
 
@@ -137,6 +138,7 @@ impl GenerateCI {
         &self,
         project_name: &str,
         bridge_model: &BridgeModel,
+        sdist: bool,
     ) -> Result<String> {
         let is_abi3 = matches!(bridge_model, BridgeModel::BindingsAbi3(..));
         let is_bin = bridge_model.is_bin();
@@ -149,16 +151,36 @@ impl GenerateCI {
                     | BridgeModel::Cffi
                     | BridgeModel::UniFfi
             );
+        let gen_cmd = std::env::args()
+            .enumerate()
+            .map(|(i, arg)| {
+                if i == 0 {
+                    env!("CARGO_PKG_NAME").to_string()
+                } else {
+                    arg
+                }
+            })
+            .collect::<Vec<String>>()
+            .join(" ");
         let mut conf = format!(
-            "# Generated by maturin v{version}
+            "# This file is autogenerated by maturin v{version}
+# To update, run
+#
+#    {gen_cmd}
+#
 on:
   push:
     branches:
       - main
       - master
+    tags:
+      - '*'
   pull_request:
   workflow_dispatch:
 
+permissions:
+  contents: read
+
 jobs:\n",
             version = env!("CARGO_PKG_VERSION"),
         );
@@ -270,6 +292,7 @@ jobs:\n",
         with:
           target: {maturin_target}
           args: --release --out dist{maturin_args}
+          sccache: 'true'
 "
             ));
             if matches!(platform, Platform::Linux) {
@@ -347,7 +370,7 @@ jobs:\n",
           source .venv/bin/activate
           pip install {project_name} --find-links dist --force-reinstall
           pip install pytest
-          {chdir} python -m pytest
+          {chdir}python -m pytest
 "
                     ));
                 } else {
@@ -368,26 +391,75 @@ jobs:\n",
             conf.push('\n');
         }
 
+        // build sdist
+        if sdist {
+            needs.push("sdist".to_string());
+
+            let maturin_args = self
+                .manifest_path
+                .as_ref()
+                .map(|manifest_path| {
+                    if manifest_path != Path::new("Cargo.toml") {
+                        format!(" --manifest-path {}", manifest_path.display())
+                    } else {
+                        String::new()
+                    }
+                })
+                .unwrap_or_default();
+            conf.push_str(&format!(
+                r#"  sdist:
+    runs-on: ubuntu-latest
+    steps:
+      - uses: actions/checkout@v3
+      - name: Build sdist
+        uses: PyO3/maturin-action@v1
+        with:
+          command: sdist
+          args: --out dist{maturin_args}
+"#
+            ));
+            conf.push_str(
+                "      - name: Upload sdist
+        uses: actions/upload-artifact@v3
+        with:
+          name: wheels
+          path: dist
+",
+            );
+            conf.push('\n');
+        }
+
         conf.push_str(&format!(
             r#"  release:
     name: Release
     runs-on: ubuntu-latest
     if: "startsWith(github.ref, 'refs/tags/')"
     needs: [{needs}]
-    steps:
+"#,
+            needs = needs.join(", ")
+        ));
+        if platforms.contains(&Platform::Emscripten) {
+            conf.push_str(
+                r#"    permissions:
+      # Used to upload release artifacts
+      contents: write
+"#,
+            );
+        }
+        conf.push_str(
+            r#"    steps:
       - uses: actions/download-artifact@v3
         with:
           name: wheels
       - name: Publish to PyPI
         uses: PyO3/maturin-action@v1
         env:
-          MATURIN_PYPI_TOKEN: ${{{{ secrets.PYPI_API_TOKEN }}}}
+          MATURIN_PYPI_TOKEN: ${{ secrets.PYPI_API_TOKEN }}
         with:
           command: upload
           args: --skip-existing *
 "#,
-            needs = needs.join(", ")
-        ));
+        );
         if platforms.contains(&Platform::Emscripten) {
             conf.push_str(
                 "      - uses: actions/download-artifact@v3
@@ -426,10 +498,14 @@ mod tests {
     #[test]
     fn test_generate_github() {
         let conf = GenerateCI::default()
-            .generate_github("example", &BridgeModel::Bindings("pyo3".to_string(), 7))
+            .generate_github(
+                "example",
+                &BridgeModel::Bindings("pyo3".to_string(), 7),
+                true,
+            )
             .unwrap()
             .lines()
-            .skip(1)
+            .skip(5)
             .collect::<Vec<_>>()
             .join("\n");
         let expected = indoc! {r#"
@@ -438,9 +514,14 @@ mod tests {
                 branches:
                   - main
                   - master
+                tags:
+                  - '*'
               pull_request:
               workflow_dispatch:
 
+            permissions:
+              contents: read
+
             jobs:
               linux:
                 runs-on: ubuntu-latest
@@ -457,6 +538,7 @@ mod tests {
                     with:
                       target: ${{ matrix.target }}
                       args: --release --out dist --find-interpreter
+                      sccache: 'true'
                       manylinux: auto
                   - name: Upload wheels
                     uses: actions/upload-artifact@v3
@@ -480,6 +562,7 @@ mod tests {
                     with:
                       target: ${{ matrix.target }}
                       args: --release --out dist --find-interpreter
+                      sccache: 'true'
                   - name: Upload wheels
                     uses: actions/upload-artifact@v3
                     with:
@@ -501,17 +584,33 @@ mod tests {
                     with:
                       target: ${{ matrix.target }}
                       args: --release --out dist --find-interpreter
+                      sccache: 'true'
                   - name: Upload wheels
                     uses: actions/upload-artifact@v3
                     with:
                       name: wheels
                       path: dist
 
+              sdist:
+                runs-on: ubuntu-latest
+                steps:
+                  - uses: actions/checkout@v3
+                  - name: Build sdist
+                    uses: PyO3/maturin-action@v1
+                    with:
+                      command: sdist
+                      args: --out dist
+                  - name: Upload sdist
+                    uses: actions/upload-artifact@v3
+                    with:
+                      name: wheels
+                      path: dist
+
               release:
                 name: Release
                 runs-on: ubuntu-latest
                 if: "startsWith(github.ref, 'refs/tags/')"
-                needs: [linux, windows, macos]
+                needs: [linux, windows, macos, sdist]
                 steps:
                   - uses: actions/download-artifact@v3
                     with:
@@ -530,10 +629,10 @@ mod tests {
     #[test]
     fn test_generate_github_abi3() {
         let conf = GenerateCI::default()
-            .generate_github("example", &BridgeModel::BindingsAbi3(3, 7))
+            .generate_github("example", &BridgeModel::BindingsAbi3(3, 7), false)
             .unwrap()
             .lines()
-            .skip(1)
+            .skip(5)
             .collect::<Vec<_>>()
             .join("\n");
         let expected = indoc! {r#"
@@ -542,9 +641,14 @@ mod tests {
                 branches:
                   - main
                   - master
+                tags:
+                  - '*'
               pull_request:
               workflow_dispatch:
 
+            permissions:
+              contents: read
+
             jobs:
               linux:
                 runs-on: ubuntu-latest
@@ -561,6 +665,7 @@ mod tests {
                     with:
                       target: ${{ matrix.target }}
                       args: --release --out dist
+                      sccache: 'true'
                       manylinux: auto
                   - name: Upload wheels
                     uses: actions/upload-artifact@v3
@@ -584,6 +689,7 @@ mod tests {
                     with:
                       target: ${{ matrix.target }}
                       args: --release --out dist
+                      sccache: 'true'
                   - name: Upload wheels
                     uses: actions/upload-artifact@v3
                     with:
@@ -605,6 +711,7 @@ mod tests {
                     with:
                       target: ${{ matrix.target }}
                       args: --release --out dist
+                      sccache: 'true'
                   - name: Upload wheels
                     uses: actions/upload-artifact@v3
                     with:
@@ -639,10 +746,14 @@ mod tests {
             ..Default::default()
         };
         let conf = gen
-            .generate_github("example", &BridgeModel::Bindings("pyo3".to_string(), 7))
+            .generate_github(
+                "example",
+                &BridgeModel::Bindings("pyo3".to_string(), 7),
+                true,
+            )
             .unwrap()
             .lines()
-            .skip(1)
+            .skip(5)
             .collect::<Vec<_>>()
             .join("\n");
         let expected = indoc! {r#"
@@ -651,9 +762,14 @@ mod tests {
                 branches:
                   - main
                   - master
+                tags:
+                  - '*'
               pull_request:
               workflow_dispatch:
 
+            permissions:
+              contents: read
+
             jobs:
               linux:
                 runs-on: ubuntu-latest
@@ -670,6 +786,7 @@ mod tests {
                     with:
                       target: ${{ matrix.target }}
                       args: --release --out dist --find-interpreter --zig
+                      sccache: 'true'
                       manylinux: auto
                   - name: Upload wheels
                     uses: actions/upload-artifact@v3
@@ -716,6 +833,7 @@ mod tests {
                     with:
                       target: ${{ matrix.target }}
                       args: --release --out dist --find-interpreter
+                      sccache: 'true'
                   - name: Upload wheels
                     uses: actions/upload-artifact@v3
                     with:
@@ -745,6 +863,7 @@ mod tests {
                     with:
                       target: ${{ matrix.target }}
                       args: --release --out dist --find-interpreter
+                      sccache: 'true'
                   - name: Upload wheels
                     uses: actions/upload-artifact@v3
                     with:
@@ -759,11 +878,26 @@ mod tests {
                       pip install pytest
                       pytest
 
+              sdist:
+                runs-on: ubuntu-latest
+                steps:
+                  - uses: actions/checkout@v3
+                  - name: Build sdist
+                    uses: PyO3/maturin-action@v1
+                    with:
+                      command: sdist
+                      args: --out dist
+                  - name: Upload sdist
+                    uses: actions/upload-artifact@v3
+                    with:
+                      name: wheels
+                      path: dist
+
               release:
                 name: Release
                 runs-on: ubuntu-latest
                 if: "startsWith(github.ref, 'refs/tags/')"
-                needs: [linux, windows, macos]
+                needs: [linux, windows, macos, sdist]
                 steps:
                   - uses: actions/download-artifact@v3
                     with:
@@ -782,10 +916,10 @@ mod tests {
     #[test]
     fn test_generate_github_bin_no_binding() {
         let conf = GenerateCI::default()
-            .generate_github("example", &BridgeModel::Bin(None))
+            .generate_github("example", &BridgeModel::Bin(None), true)
             .unwrap()
             .lines()
-            .skip(1)
+            .skip(5)
             .collect::<Vec<_>>()
             .join("\n");
         let expected = indoc! {r#"
@@ -794,9 +928,14 @@ mod tests {
                 branches:
                   - main
                   - master
+                tags:
+                  - '*'
               pull_request:
               workflow_dispatch:
 
+            permissions:
+              contents: read
+
             jobs:
               linux:
                 runs-on: ubuntu-latest
@@ -810,6 +949,7 @@ mod tests {
                     with:
                       target: ${{ matrix.target }}
                       args: --release --out dist
+                      sccache: 'true'
                       manylinux: auto
                   - name: Upload wheels
                     uses: actions/upload-artifact@v3
@@ -829,6 +969,7 @@ mod tests {
                     with:
                       target: ${{ matrix.target }}
                       args: --release --out dist
+                      sccache: 'true'
                   - name: Upload wheels
                     uses: actions/upload-artifact@v3
                     with:
@@ -847,17 +988,33 @@ mod tests {
                     with:
                       target: ${{ matrix.target }}
                       args: --release --out dist
+                      sccache: 'true'
                   - name: Upload wheels
                     uses: actions/upload-artifact@v3
                     with:
                       name: wheels
                       path: dist
 
+              sdist:
+                runs-on: ubuntu-latest
+                steps:
+                  - uses: actions/checkout@v3
+                  - name: Build sdist
+                    uses: PyO3/maturin-action@v1
+                    with:
+                      command: sdist
+                      args: --out dist
+                  - name: Upload sdist
+                    uses: actions/upload-artifact@v3
+                    with:
+                      name: wheels
+                      path: dist
+
               release:
                 name: Release
                 runs-on: ubuntu-latest
                 if: "startsWith(github.ref, 'refs/tags/')"
-                needs: [linux, windows, macos]
+                needs: [linux, windows, macos, sdist]
                 steps:
                   - uses: actions/download-artifact@v3
                     with:
