@@ -1,3 +1,5 @@
+//! A pyproject.toml as specified in PEP 517
+
 use crate::PlatformTag;
 use anyhow::{format_err, Result};
 use fs_err as fs;
@@ -12,12 +14,82 @@ pub struct Tool {
     maturin: Option<ToolMaturin>,
 }
 
+#[derive(Copy, Clone, Debug, Deserialize, Serialize, PartialEq, Eq)]
+#[serde(rename_all = "lowercase")]
+/// The target format for the include or exclude [GlobPattern].
+///
+/// See [Formats].
+pub enum Format {
+    /// Source distribution
+    Sdist,
+    /// Wheel
+    Wheel,
+}
+
+#[derive(Clone, Debug, Deserialize, Serialize, PartialEq, Eq)]
+#[serde(untagged)]
+/// A single [Format] or multiple [Format] values for a [GlobPattern].
+pub enum Formats {
+    /// A single [Format] value
+    Single(Format),
+    /// Multiple [Format] values
+    Multiple(Vec<Format>),
+}
+
+impl Formats {
+    /// Returns `true` if the inner [Format] value(s) target the given [Format]
+    pub fn targets(&self, format: Format) -> bool {
+        match self {
+            Self::Single(val) if val == &format => true,
+            Self::Multiple(formats) if formats.contains(&format) => true,
+            _ => false,
+        }
+    }
+}
+
+#[derive(Clone, Debug, Deserialize, Serialize, PartialEq, Eq)]
+#[serde(untagged)]
+/// A glob pattern for the include and exclude configuration.
+///
+/// See [PyProjectToml::include] and [PyProject::exclude].
+///
+/// Based on <https://python-poetry.org/docs/pyproject/#include-and-exclude>.
+pub enum GlobPattern {
+    /// A glob
+    Path(String),
+    /// A glob `path` with a `format` key to specify one or more [Format] values
+    WithFormat {
+        /// A glob
+        path: String,
+        /// One or more [Format] values
+        format: Formats,
+    },
+}
+
+impl GlobPattern {
+    /// Returns the glob pattern for this pattern if it targets the given [Format], else this returns `None`.
+    pub fn targets(&self, format: Format) -> Option<&str> {
+        match self {
+            // Not specified defaults to both
+            Self::Path(ref glob) => Some(glob),
+            Self::WithFormat {
+                path,
+                format: formats,
+            } if formats.targets(format) => Some(path),
+            _ => None,
+        }
+    }
+}
+
 /// The `[tool.maturin]` section of a pyproject.toml
 #[derive(Serialize, Deserialize, Debug, Clone)]
 #[serde(rename_all = "kebab-case")]
 pub struct ToolMaturin {
     // maturin specific options
+    // TODO(0.15.0): remove deprecated
     sdist_include: Option<Vec<String>>,
+    include: Option<Vec<GlobPattern>>,
+    exclude: Option<Vec<GlobPattern>>,
     bindings: Option<String>,
     #[serde(alias = "manylinux")]
     compatibility: Option<PlatformTag>,
@@ -27,6 +99,8 @@ pub struct ToolMaturin {
     strip: bool,
     /// The directory with python module, contains `<module_name>/__init__.py`
     python_source: Option<PathBuf>,
+    /// Python packages to include
+    python_packages: Option<Vec<String>>,
     /// Path to the wheel directory, defaults to `<module_name>.data`
     data: Option<PathBuf>,
     // Some customizable cargo options
@@ -48,6 +122,7 @@ pub struct ToolMaturin {
     pub config: Option<Vec<String>>,
     /// Unstable (nightly-only) flags to Cargo, see 'cargo -Z help' for details
     pub unstable_flags: Option<Vec<String>>,
+    /// Additional rustc arguments
     pub rustc_args: Option<Vec<String>>,
 }
 
@@ -80,12 +155,17 @@ impl PyProjectToml {
     /// source distributions
     pub fn new(pyproject_file: impl AsRef<Path>) -> Result<PyProjectToml> {
         let path = pyproject_file.as_ref();
-        let contents = fs::read_to_string(&path)?;
+        let contents = fs::read_to_string(path)?;
         let pyproject: PyProjectToml = toml_edit::easy::from_str(&contents)
             .map_err(|err| format_err!("pyproject.toml is not PEP 517 compliant: {}", err))?;
         Ok(pyproject)
     }
 
+    /// Returns the value of `[project.name]` in pyproject.toml
+    pub fn project_name(&self) -> Option<&str> {
+        self.project.as_ref().map(|project| project.name.as_str())
+    }
+
     /// Returns the values of `[tool.maturin]` in pyproject.toml
     #[inline]
     pub fn maturin(&self) -> Option<&ToolMaturin> {
@@ -93,10 +173,24 @@ impl PyProjectToml {
     }
 
     /// Returns the value of `[tool.maturin.sdist-include]` in pyproject.toml
+    #[deprecated(
+        since = "0.14.0",
+        note = "please use `PyProjectToml::include` (<https://github.com/PyO3/maturin/pulls/1255>)"
+    )]
     pub fn sdist_include(&self) -> Option<&Vec<String>> {
         self.maturin()?.sdist_include.as_ref()
     }
 
+    /// Returns the value of `[tool.maturin.include]` in pyproject.toml
+    pub fn include(&self) -> Option<&[GlobPattern]> {
+        self.maturin()?.include.as_ref().map(AsRef::as_ref)
+    }
+
+    /// Returns the value of `[tool.maturin.exclude]` in pyproject.toml
+    pub fn exclude(&self) -> Option<&[GlobPattern]> {
+        self.maturin()?.exclude.as_ref().map(AsRef::as_ref)
+    }
+
     /// Returns the value of `[tool.maturin.bindings]` in pyproject.toml
     pub fn bindings(&self) -> Option<&str> {
         self.maturin()?.bindings.as_deref()
@@ -127,6 +221,12 @@ impl PyProjectToml {
             .and_then(|maturin| maturin.python_source.as_deref())
     }
 
+    /// Returns the value of `[tool.maturin.python-packages]` in pyproject.toml
+    pub fn python_packages(&self) -> Option<&[String]> {
+        self.maturin()
+            .and_then(|maturin| maturin.python_packages.as_deref())
+    }
+
     /// Returns the value of `[tool.maturin.data]` in pyproject.toml
     pub fn data(&self) -> Option<&Path> {
         self.maturin().and_then(|maturin| maturin.data.as_deref())
@@ -177,8 +277,7 @@ impl PyProjectToml {
         if self.build_system.build_backend.as_deref() != Some(maturin) {
             eprintln!(
                 "⚠️  Warning: `build-backend` in pyproject.toml is not set to `{maturin}`, \
-                    packaging tools such as pip will not use maturin to build this project.",
-                maturin = maturin
+                    packaging tools such as pip will not use maturin to build this project."
             );
             return false;
         }
@@ -188,7 +287,10 @@ impl PyProjectToml {
 
 #[cfg(test)]
 mod tests {
-    use crate::PyProjectToml;
+    use crate::{
+        pyproject_toml::{Format, Formats, GlobPattern, ToolMaturin},
+        PyProjectToml,
+    };
     use fs_err as fs;
     use pretty_assertions::assert_eq;
     use std::path::Path;
@@ -207,6 +309,7 @@ mod tests {
 
             [tool.maturin]
             manylinux = "2010"
+            python-packages = ["foo", "bar"]
             manifest-path = "Cargo.toml"
             profile = "dev"
             features = ["foo", "bar"]
@@ -233,6 +336,10 @@ mod tests {
             maturin.rustc_args,
             Some(vec!["-Z".to_string(), "unstable-options".to_string()])
         );
+        assert_eq!(
+            maturin.python_packages,
+            Some(vec!["foo".to_string(), "bar".to_string()])
+        );
     }
 
     #[test]
@@ -256,4 +363,66 @@ mod tests {
         let without_constraint = PyProjectToml::new(pyproject_file).unwrap();
         assert!(!without_constraint.warn_missing_maturin_version());
     }
+
+    #[test]
+    fn deserialize_include_exclude() {
+        let single = r#"include = ["single"]"#;
+        assert_eq!(
+            toml_edit::easy::from_str::<ToolMaturin>(single)
+                .unwrap()
+                .include,
+            Some(vec![GlobPattern::Path("single".to_string())])
+        );
+
+        let multiple = r#"include = ["one", "two"]"#;
+        assert_eq!(
+            toml_edit::easy::from_str::<ToolMaturin>(multiple)
+                .unwrap()
+                .include,
+            Some(vec![
+                GlobPattern::Path("one".to_string()),
+                GlobPattern::Path("two".to_string())
+            ])
+        );
+
+        let single_format = r#"include = [{path = "path", format="sdist"}]"#;
+        assert_eq!(
+            toml_edit::easy::from_str::<ToolMaturin>(single_format)
+                .unwrap()
+                .include,
+            Some(vec![GlobPattern::WithFormat {
+                path: "path".to_string(),
+                format: Formats::Single(Format::Sdist)
+            },])
+        );
+
+        let multiple_formats = r#"include = [{path = "path", format=["sdist", "wheel"]}]"#;
+        assert_eq!(
+            toml_edit::easy::from_str::<ToolMaturin>(multiple_formats)
+                .unwrap()
+                .include,
+            Some(vec![GlobPattern::WithFormat {
+                path: "path".to_string(),
+                format: Formats::Multiple(vec![Format::Sdist, Format::Wheel])
+            },])
+        );
+
+        let mixed = r#"include = ["one", {path = "two", format="sdist"}, {path = "three", format=["sdist", "wheel"]}]"#;
+        assert_eq!(
+            toml_edit::easy::from_str::<ToolMaturin>(mixed)
+                .unwrap()
+                .include,
+            Some(vec![
+                GlobPattern::Path("one".to_string()),
+                GlobPattern::WithFormat {
+                    path: "two".to_string(),
+                    format: Formats::Single(Format::Sdist),
+                },
+                GlobPattern::WithFormat {
+                    path: "three".to_string(),
+                    format: Formats::Multiple(vec![Format::Sdist, Format::Wheel])
+                }
+            ])
+        );
+    }
 }
