@@ -20,12 +20,16 @@ else:
     ext_suffix = sysconfig.get_config_var("EXT_SUFFIX")
 
 metadata = {
+    # sys.implementation.name can differ from platform.python_implementation(), for example
+    # Pyston has sys.implementation.name == "pyston" while platform.python_implementation() == cpython
+    "implementation_name": sys.implementation.name,
     "executable": sys.executable or None,
     "major": sys.version_info.major,
     "minor": sys.version_info.minor,
     "abiflags": sysconfig.get_config_var("ABIFLAGS"),
     "interpreter": platform.python_implementation().lower(),
     "ext_suffix": ext_suffix,
+    "soabi": sysconfig.get_config_var("SOABI") or None,
     "abi_tag": (sysconfig.get_config_var("SOABI") or "-").split("-")[1] or None,
     "platform": sysconfig.get_platform(),
     # This one isn't technically necessary, but still very useful for sanity checks
